/*
  This file is part of the KDE libraries
  Copyright (c) 2005 Willem Verschuur <willverschuur@yahoo.com>
  Copyright (c) 2006 Helio Chissini de Castro <helio@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

#include <iostream>

#include "kconfigldapbackend.h"
#include "kconfigdata.h"
#include "kuser.h"
#include "xldaplite.h"

KConfigLDAPBackEnd::KConfigLDAPBackEnd( KConfigBase *config, 
                                        const QString &fileName,
                                        const char * resType, 
                                        bool useKDEGlobals )

    :   KConfigBackEnd(config, fileName, resType, useKDEGlobals)

{
   _fileImmutable = false;
}




KConfigLDAPBackEnd::~KConfigLDAPBackEnd()
{
}



bool KConfigLDAPBackEnd::parseConfigFiles()
{
    // will result in an invalid search anyway..
    if (mfileName.isEmpty()) 
       return false;

    // blacklists some entries to avoid uselless queries
    if (  mfileName == "kdebugrc" 
          || mfileName == "kconf_updaterc" 
          || mfileName == "kdedrc"
          || mfileName == "kdeinitrc"
          || mfileName == "kbuildsycocarc"
          || mfileName.startsWith( "/" ) )
       return false;

    // connect to ldap server
    xLDAPconnection ldap;
    if (!ldap.connect()) return false;

    // User and profilelist
    KUser user;
    QString basedn = QString( "cn=%1,ou=default,ou=KDEConfig,%2" ).arg( mfileName ).arg( ldap.base() );
    QStringList profiles( basedn );

    // Query group profile
    basedn = QString("uid=%1,ou=People,%2").arg(user.loginName()).arg(ldap.base());
    char *attrs[] = { "kdeConfigEntry", 0 };
    xLDAPquery query( ldap, "(objectClass=posixAccount)", basedn, attrs );

    for ( xLDAPiterator section(query); section; ++section )
    {
		 for ( xLDAPattribute gattr(section); gattr; ++gattr )
		 {
          basedn = QString( "cn=%1,%2" ).arg( mfileName ).arg( gattr.firstValue() );
          profiles.push_front( basedn );
		 }
    }
    
    for ( QStringList::Iterator it = profiles.begin(); it != profiles.end(); ++it ) 
    {
       // File immutable
       xLDAPquery filequery( ldap, "(objectClass=appConfig)", *it );
       for ( xLDAPiterator section( filequery ); section; ++section )
       {
          for ( xLDAPattribute gattr(section); gattr; ++gattr )
          {
             if (!strcmp(gattr.attribute(), "immutable")) // immutable
                setFileImmutable(0 == strcmp(gattr.firstValue(), "TRUE"));
          }
       }
       
       xLDAPquery query( ldap, "(objectClass=appConfigSection)", *it );
       
       // write results to application settings 
       // iterate through sections/groups
       for ( xLDAPiterator section(query); section; ++section )
       {
          QCString aCurrentGroup = "<default>";
          bool groupimmutable = false;
          // find out group name and flags
          for ( xLDAPattribute gattr(section); gattr; ++gattr )
          {
             // section name
             if (!strcmp(gattr.attribute(), "cn"))
                aCurrentGroup = gattr.firstValue();
             else if (!strcmp(gattr.attribute(), "immutable"))
             {
                if ( getFileImmutable() )
                   groupimmutable = true;
                else
                   groupimmutable = (0 == strcmp(gattr.firstValue(), "TRUE"));
             }
             if ( groupimmutable )
                setGroupImmutable( aCurrentGroup, true );
          }
          
          // backwards compatibility
          if (aCurrentGroup == "KDE Desktop Entry")
             aCurrentGroup = "Desktop Entry";
          
          KEntryKey groupKey(aCurrentGroup, 0);
          KEntry entry = pConfig->lookupData(groupKey);
          bool groupSkip = entry.bImmutable;
          
          if (groupSkip)
             continue;
          
          entry.bImmutable |= groupimmutable;
          pConfig->putData(groupKey, entry, false);
  
          // set attribute values
          //std::cout << "LDAP: " << "[ " << aCurrentGroup << " ] " <<  std::endl;
          for ( xLDAPattribute attr(section);  attr; ++attr )
          {
             if (strcmp(attr.attribute(), "appconfigentry")) continue;
             for ( char* cattr = (char*) attr.firstValue();
                   cattr;
                   cattr = (char*) attr.nextValue() )
             {
                QCString cattribute((const char*) cattr); 
                cattr = cattribute.data();
                char* ceq = (char*) strchr(cattr, '=');
                if (!ceq) 
                   continue;
                
                *ceq = 0;
                char* cvalue = ++ceq;
                
                // insert the key/value line
                KEntryKey aEntryKey(aCurrentGroup, cattr);
                aEntryKey.bLocal = false; //(locale != 0);
                aEntryKey.bDefault = false; //bDefault; meep?
                
                //std::cout << "LDAP: " << cattr << " = " << cvalue <<  std::endl;
                
                KEntry aEntry;
                aEntry.mValue = cvalue;
                aEntry.bGlobal = true; //bGlobal;
                aEntry.bImmutable = false; //optionImmutable;
                aEntry.bDeleted = false; // optionDeleted;
                aEntry.bExpand = false; // optionExpand;
                aEntry.bNLS = false; //(locale != 0);
                
                pConfig->putData( aEntryKey, // group
                      aEntry,    // entry
                      false );   
             }
          }
       }
    }
    // return successful
    return true;
}



void KConfigLDAPBackEnd::sync(bool bMerge)
{
    // write back to ldap server..
}



void KConfigLDAPBackEnd::virtual_hook(int id, void* data)
{
    KConfigBackEnd::virtual_hook( id, data );
}

