/*
  This file is part of the KDE libraries
  Copyright (c) 2005 Willem Verschuur <willverschuur@yahoo.com>
  Copyright (c) 2006 Helio Chissini de Castro <helio@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

#include <qfile.h>
#include <qmap.h>
#include <qregexp.h>
#include <unistd.h>
#include <iostream>

#include "xldaplite.h"

#ifndef LDAP_CONF
#define LDAP_CONF "/etc/ldap.conf"
#endif

#ifndef LDAP_SECRET
#define LDAP_SECRET "/etc/ldap.secret"
#endif

xLDAPconnection::xLDAPconnection()
{
    
    m_Connection = 0;
    
}



xLDAPconnection::~xLDAPconnection()
{
    if (m_Connection)
    {
        disconnect();
    }
}



bool xLDAPconnection::connect()
{

    // close existing connection
    if (m_Connection)
    {
        disconnect();
    }

    QFile conf(LDAP_CONF);
    QMap<QString,QString> config;
  
    if (!conf.open(IO_ReadOnly)) 
    {
        //std::cerr << "Cannot open " << LDAP_CONF << std::endl;
        return false;
    }

    QTextStream confs(&conf);
    
    while (!confs.atEnd()) 
    {
        QString line = confs.readLine();
        if (line.contains(QRegExp("^\\s*[a-z]+\\s+.+"))) 
        {
            QStringList strs=QStringList::split(QRegExp("\\s+"),line);
            config[strs[0]] = strs[1];
        }
    }
    
    if (geteuid()==0) 
    {
        
        QFile secretFile( LDAP_SECRET );
        
        if (!secretFile.open(IO_ReadOnly)) 
        {
            std::cerr << "Cannot find " << LDAP_SECRET << std::endl; 
            return false;
        }
        
        QTextStream secretStream(&secretFile);
        m_RootPassword = secretStream.readLine();
        
    }

    // configure parameters 
    QString host = config["host"];
    int port = 389;
    
    if (config.contains("port")) 
    {
        port = config["port"].toInt();
    }
    
    // create ldap connection
    m_Connection = ldap_init(host.latin1(), port);

    if (!m_Connection) return false;

    int version = LDAP_VERSION3;
    
    if (config.contains("ldap_version"))
        version = config["ldap_version"].toInt();
   
    // configure connection
    int ret;
    if ((ret = ldap_set_option(m_Connection, LDAP_OPT_PROTOCOL_VERSION, &version)) != LDAP_SUCCESS) 
    {
        std::cerr << "ldap_set_option" << ldap_err2string(ret) << std::endl;
        return false;
    } 

    // root binding
    if (getuid==0 && config.contains("rootbinddn")) 
    {
        if ( LDAP_SUCCESS != ldap_bind_s( m_Connection,
                                          config["rootbinddn"].latin1(),
                                          m_RootPassword.latin1(), 
                                          LDAP_AUTH_SIMPLE ) )
        {
            std::cerr << "Root bind failed" << std::endl;
            return false;
        }
        // should be utf8....
    } 
    else 
    {
        if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(m_Connection,NULL,NULL)))
        {
            std::cerr << "Anonymous bind failed" << std::endl;
            return false;
        }
        // there are also SASL, and kerberos
    }
    
    // configure scope
    m_Scope = LDAP_SCOPE_SUBTREE;
    
    if (config.contains("scope")) 
    {
        if (config["scope"] == "sub");
        else if (config["scope"] == "one")
            m_Scope = LDAP_SCOPE_ONELEVEL;
        else if (config["scope"] == "base")
            m_Scope = LDAP_SCOPE_BASE;
        else 
        {
            std::cerr << "bad value for scope in ldap.conf: " << config["scope"].latin1() << std::endl;
            return false;
        }
    }
    
    // configure base
    if (!config.contains("base")) 
    {
        std::cerr << "base is not defined in ldap.conf" << std::endl;
        return false;
    }
    
    m_Base = config["base"];

    // return successful
    return true;

}



void xLDAPconnection::disconnect()
{

    // the connection doesnt exist
    if (!m_Connection) return;

    // drop connection
    ldap_unbind(m_Connection);
    m_Connection = 0;

    return;

}



xLDAPquery::xLDAPquery( xLDAPconnection& connection,
                        const QString& filter,
                        const QString& pbasedn,
                        char** attributes )
{

    // initialize vars
    QString basedn;
    
    if (pbasedn.isNull())
    {
        basedn = connection.m_Base;
    }
    else
    {
        basedn = pbasedn;
    }
    
    m_Connection = &connection;

    // execute search
    m_Result = ldap_search_s(   m_Connection->m_Connection,
                                basedn.latin1(),
                                m_Connection->m_Scope,
                                filter.latin1(),   
                                attributes,
                                0, // return attrs & values
                                &m_Query   );

    // output an error if query failed
    //if (m_Result != LDAP_SUCCESS) 
  	    //std::cerr << "ldap_search: " << ldap_err2string(m_Result) << std::endl;

}



xLDAPquery::~xLDAPquery()
{
    if (m_Query)
    {
        ldap_msgfree(m_Query);
        m_Query = 0;
    }
}



xLDAPiterator::xLDAPiterator(xLDAPquery& query)
{
    
    // initialize vars
    m_Query = &query;
    m_Entry = 0;   // used by element iterator (this)
    m_Element = 0; // used by attribute iterator

    // fix error handling 
    // if (!m_Connection) return false;
    // if (!m_Message) return false;
    
    // point to first element in query result
    m_Entry = ldap_first_entry(m_Query->m_Connection->m_Connection, m_Query->m_Query);
    
}



xLDAPiterator::~xLDAPiterator()
{

    clearElement();
}



void xLDAPiterator::clearElement()
{

    if (m_Element)
    {
        ber_free(m_Element, 0);
        m_Element = 0;
    }

}



xLDAPiterator::operator bool ()
{
    return (m_Entry != 0);
}



void xLDAPiterator::operator ++ ()
{
    m_Entry = ldap_next_entry(m_Query->m_Connection->m_Connection, m_Entry);
}



xLDAPattribute::xLDAPattribute(xLDAPiterator& iter)
{

    // initialize class vars
    m_Iterator = &iter;
    m_Attribute = 0;
    m_Values = 0;
    m_CurrentValue = 0;

    // free previous element's attribute data 
    if (m_Iterator->m_Element)
    {
        m_Iterator->clearElement();
    }

    // get first attribute
    m_Attribute = ldap_first_attribute( m_Iterator->m_Query->m_Connection->m_Connection, 
                                        m_Iterator->m_Entry, 
                                        &m_Iterator->m_Element);

}



xLDAPattribute::~xLDAPattribute()
{
    clearAttribute();
}



void xLDAPattribute::clearAttribute()
{

    if (m_Attribute)
    {
        ldap_memfree(m_Attribute);
        m_Attribute = 0;
    }
    
    if (m_Values)
    {
        clearValues();
    }

}



xLDAPattribute::operator bool ()
{
    return (m_Attribute != 0);
}



void xLDAPattribute::operator ++ ()
{

    clearAttribute();
    m_Attribute = ldap_next_attribute( m_Iterator->m_Query->m_Connection->m_Connection, 
                                       m_Iterator->m_Entry, 
                                       m_Iterator->m_Element );

}



const char* xLDAPattribute::firstValue()
{
    if (m_Values)
    {
        clearValues();
    }

    m_Values = ldap_get_values (  m_Iterator->m_Query->m_Connection->m_Connection, 
                                  m_Iterator->m_Entry, 
                                  m_Attribute );

    m_CurrentValue = m_Values;
    return *m_CurrentValue;
}


const char* xLDAPattribute::nextValue()
{
    m_CurrentValue++;
    return *m_CurrentValue;
}



void xLDAPattribute::clearValues()
{
    if (m_Values)
    {
        ldap_value_free(m_Values);
        m_Values = 0;
    }
}

